import 'dart:convert';
import 'dart:io';

import 'package:flutter_vizzy_lib/_Md5Util.dart';
import 'package:flutter_vizzy_lib/_VizzyApiClient.dart';
import 'package:flutter_vizzy_lib/_VizzyService.dart';
import 'package:flutter_driver/flutter_driver.dart';
import 'package:meta/meta.dart';

class Vizzy {

  String _userEmail;
  String _userToken;
  String _vizzyUrl; // Important to finish with /
  int _projectId;

  String _commitSha;
  String _pullRequestNumber;

  var _imagesDir = Directory("vizzy");

  FlutterDriver _driver;

  VizzyService _apiService;

  Vizzy({@required FlutterDriver driver, @required int projectId, String url, String username, String token}) {
    _driver = driver;
    _initializeParameters(url, username, token);
    _projectId = projectId;
    _apiService = VizzyService(
        baseUrl: _vizzyUrl,
        userEmail: _userEmail,
        userToken: _userToken);
    _prepareScreenshotDir();
  }

  void _initializeParameters(String url, String username, String token) {
    // Environment variables seems to be the easiest way
    // Defaulting for now to our configuration
    // In the future, finish with an error if not configured
    _userEmail = username ?? Platform.environment['VIZZY_EMAIL'];
    _userToken = token ?? Platform.environment['VIZZY_TOKEN'];
    _vizzyUrl = url ?? Platform.environment['VIZZY_URL'];

    _commitSha = Platform.environment['COMMIT_SHA'];
    _pullRequestNumber = Platform.environment['PULL_REQUEST_NUMBER'];
    // If there is a PR number, it is not a dev build
  }

  void _prepareScreenshotDir() {
    if (_imagesDir.existsSync()) {
      _imagesDir.deleteSync(recursive: true);
    }
    _imagesDir.createSync(recursive: true);
  }

  Future takeScreenshot([String screenshotName]) async {
    final filePath = "${_imagesDir.path}/$screenshotName.png";
    final file = await File(filePath).create(recursive: true);
    final pixels = await _driver.screenshot();
    await file.writeAsBytes(pixels);
  }

  Future verifyImages() async {
    BuildInfo buildInfo = await _apiService.createBuild(_projectId, _commitSha, _pullRequestNumber);

    var imagesToUpload = await _filterImagesToUpload(buildInfo.id);

    await _uploadRequestedImages(buildInfo.id, imagesToUpload);

    await _apiService.commitBuild(buildInfo.id);
  }

  Future<List<String>> _filterImagesToUpload(int buildId) async {
    var imageMd5Request = await _calculateMd5s();
    var imageMd5response = await _apiService.postImageMd5s(buildId, imageMd5Request);
    return imageMd5response.imageMd5s.keys.toList();
  }

  Future<ImageMd5Request> _calculateMd5s() async {
    var allFiles = await _getAllFilesUnderData(_imagesDir);
    var imagesMap = Map<String, String>();
    for(File file in allFiles) {
      String md5 = await calculateMD5File(file);
      // We get the path from the app directory, remove that part
      var imagePath = file.path.replaceFirst("${_imagesDir.path}/", "");
      imagesMap[imagePath] = md5;
    }
    var imageMd5Request = ImageMd5Request();
    imageMd5Request.imageMd5s = json.encode(imagesMap);

    return imageMd5Request;
  }

  Future<List<File>> _getAllFilesUnderData(Directory imagesDir) async {
    var allImages = List<File>();
    await for (FileSystemEntity entity in imagesDir.list(recursive: true, followLinks: false)) {
      allImages.add(File(entity.path));
    }
    return allImages;
  }

  Future _uploadRequestedImages(int buildId, List<String> imagesToUpload) async {
    for(String fileName in imagesToUpload) {
      File file = File("${_imagesDir.path}/$fileName");
      await _apiService.uploadImage(buildId, file, fileName);
    }
  }
}
